/* childPidWatcher.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "childPidWatcher.h"

#include <signal.h>
#include <sys/wait.h>

ChildPidWatcher::Pids ChildPidWatcher::_pids;

// We create this instance to force the constructor to be called,
// which will install the signal handler.
static ChildPidWatcher t1;

ChildPidWatcher::
ChildPidWatcher() {
  // Install a signal handler so we get all SIGCHLD signals whenever
  // any child process terminates.

  struct sigaction act;
  act.sa_handler = sigchld_handler;
  sigemptyset(&act.sa_mask);
  act.sa_flags = SA_NOCLDSTOP;
  if (sigaction(SIGCHLD, &act, (struct sigaction *)NULL) < 0) {
    perror("sigaction");
    exit(1);
  }
}

ChildPidWatcher::SignalType &ChildPidWatcher::
watch_pid(pid_t pid) {
  SignalType *signal = (SignalType *)NULL;

  Pids::iterator pi;
  pi = _pids.find(pid);
  if (pi != _pids.end()) {
    signal = (*pi).second;
  } else {
    signal = new SignalType;
    _pids[pid] = signal;
  }

  return *signal;
}

void ChildPidWatcher::
signal_pid(pid_t pid, bool normal_exit, int exit_status) {
  Pids::iterator pi;
  pi = _pids.find(pid);
  if (pi != _pids.end()) {
    SignalType *signal = (*pi).second;
    (*signal)(pid, normal_exit, exit_status);
    delete signal;
    _pids.erase(pi);
  }
}

void ChildPidWatcher::
sigchld_handler(int signo) {
  // Some child just terminated.  Get its exit status.
  int status;
  pid_t pid = waitpid(-1, &status, WNOHANG);
  if (pid > 0) {
#if defined(__GNUC__)
    // Some versions of EGCS have a bad header file that misdefines
    // WIFEXITED() etc.  We have to use the lower-level
    // implementations for these.
    signal_pid(pid, __WIFEXITED(status), __WEXITSTATUS(status));
#else
    signal_pid(pid, WIFEXITED(status), WEXITSTATUS(status));
#endif
  }
}

