/* expand_variables.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"
#include "expand_variables.h"

string 
expand_variables(const string &str, const VariableDef &vars) {
  string result;

  int i = 0;
  while (i < str.length()) {
    if (str[i] != '$') {
      // Not a dollar sign, just a normal character.
      result += str[i];
      i++;

    } else {
      // A dollar sign begins a variable reference.
      i++;
      if (i < str.length() && str[i] == '$') {
	// Oh, no it doesn't.
	result += '$';
	i++;
      } else {
	string varname;

	// In case a variable reference is ambiguous, we'll store its
	// alternate interpretation in alt_varname.
	string alt_varname;
	int alt_end;

	if (str[i] == '{') {
	  // A reference like ${varname}
	  i++;
	  int j = i;
	  while (j < str.length() && str[j] != '}') {
	    j++;
	  }
	  alt_varname = varname = str.substr(i, j - i);
	  i = j+1;
	  alt_end = i;

	} else {
	  // A reference like $varname
	  int j = i;
	  while (j < str.length() && isalnum(str[j])) {
	    j++;
	  }
	  alt_varname = str.substr(i, j - i);
	  alt_end = j;
	  while (j < str.length() && 
		 (isalnum(str[j]) || str[j]=='_' || str[j]=='.')) {
	    j++;
	  }
	  varname = str.substr(i, j - i);
	  i = j;
	}

	VariableDef::const_iterator vmi;
	vmi = vars.find(varname);
	if (vmi == vars.end() && varname != alt_varname) {
	  // Hmm, it wasn't a valid variable--try the alternate
	  // interpretation.
	  vmi = vars.find(alt_varname);
	  i = alt_end;
	}

	if (vmi == vars.end()) {
	  //	  cerr << "Invalid variable reference $" << varname << "\n";
	} else {
	  // This variable name actually exists!  Hooray for the user!
	  result += (*vmi).second;
	}
      }
    }
  }

  return result;
}

