/* importList.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef IMPORTLIST_H
#define IMPORTLIST_H

// This is the list of files that are found in the import directory,
// before they have been imported into the database.  It's actually a
// 2-d array of cells, because each filename discovered is split up
// into several pieces according to the user's bracketing in the
// matching regexp.  So we end up with an array of n rows (files) by m
// columns (pieces).

#include "dataCell.h"

#include <gtk--.h>
#include <vector>
#include <string>

class ImportList {
public:
  ImportList();

  int get_num_rows() const;
  int get_num_columns() const;

  // check_min_columns() makes sure the list has at least this number
  // of columns.  If it does not, it adds some more.
  void check_min_columns(int cols);

  // make_clist() allocates and fills a CList corresponding to the
  // data stored in the ImportList.
  Gtk_CList *make_clist();

  enum ColumnType {
    CT_none,
    CT_song_name,
    CT_performer,
    CT_writer,
    CT_track_number,
    CT_cd_title,
    CT_cd_performer,
    CT_cd_writer,
    CT_num_types
  };

  static string get_type_description(ColumnType type);
  static DataCell::Type get_type_datatype(ColumnType type);

  class Row {
  public:
    Row(const string &relpath, int type);
    
    string _relpath;
    int _type;
    typedef vector<string> Data;
    Data _data;
  };
  
  class Column {
  public:
    Column(const string &heading);

    void set_type(ColumnType type);

    string _heading;
    ColumnType _type;
  };

  typedef vector<Row> Rows;
  typedef vector<Column> Columns;
  Rows _rows;
  Columns _columns;
};

#endif
